package dev.shadowsoffire.apotheosis.adventure.socket.gem;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.google.common.base.Preconditions;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import dev.shadowsoffire.apotheosis.adventure.AdventureConfig;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityClamp;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import java.util.function.Predicate;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.util.RandomSource;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.ItemLike;
import net.minecraftforge.common.crafting.conditions.ICondition;

public class GemRegistry extends WeightedDynamicRegistry<Gem> {
  public static final dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry INSTANCE = new dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry();
  
  public GemRegistry() {
    super(AdventureModule.LOGGER, "gems", true, false);
  }
  
  protected void registerBuiltinCodecs() {
    registerDefaultCodec(Apotheosis.loc("gem"), Gem.CODEC);
  }
  
  @SafeVarargs
  public static ItemStack createRandomGemStack(RandomSource rand, ServerLevel level, float luck, Predicate<Gem>... filter) {
    Gem gem = (Gem)INSTANCE.getRandomItem(rand, luck, (Predicate[])filter);
    if (gem == null)
      return ItemStack.f_41583_; 
    RarityClamp clamp = (RarityClamp)AdventureConfig.GEM_DIM_RARITIES.get(level.m_46472_().m_135782_());
    LootRarity rarity = gem.clamp(LootRarity.random(rand, luck, clamp));
    return createGemStack(gem, rarity);
  }
  
  public static ItemStack createGemStack(Gem gem, LootRarity rarity) {
    ItemStack stack = new ItemStack((ItemLike)Adventure.Items.GEM.get());
    GemItem.setGem(stack, gem);
    AffixHelper.setRarity(stack, rarity);
    return stack;
  }
  
  protected void validateItem(ResourceLocation key, Gem item) {
    super.validateItem(key, (CodecProvider)item);
    for (LootRarity r = item.minRarity; r != item.maxRarity; r = r.next()) {
      boolean atLeastOne = false;
      for (GemBonus bonus : item.bonuses) {
        if (bonus.supports(r))
          atLeastOne = true; 
      } 
      Preconditions.checkArgument(atLeastOne, "No bonuses provided for supported rarity %s. At least one bonus must be provided, or the rarity should not be supported.", RarityRegistry.INSTANCE.getKey((CodecProvider)r));
    } 
  }
  
  public final ICondition.IContext _getContext() {
    return getContext();
  }
}
